<?php
/**
 * Register "Custom CSS" submenu in "Appearance" Admin Menu
 *
 * @since 1.0
 * @todo remove delete_option
 */
function hmpl_css_register_submenu_page() {
	// param $page returns automatically
	$page = add_theme_page( __( 'Custom CSS', 'hmpl_css' ), __( 'Custom CSS', 'hmpl_css' ), 'edit_theme_options', 'custom-css', 'hmpl_css_render_submenu_page' );
	
	// Enqueue styles and scripts
	add_action( 'admin_print_styles-' . $page, 'hmpl_css_register_codemirror' );
	
	// Call register settings function
	add_action( 'admin_init', 'hmpl_css_register_settings' );
}
add_action( 'admin_menu', 'hmpl_css_register_submenu_page' );


/**
 * Enqueues Scripts/Styles for CodeMirror
 *
 * @param dir
 * @return void
 */
function hmpl_css_register_codemirror() {
		$dir = plugin_dir_path( __FILE__ );
		
		wp_enqueue_style( 'codemirror-css', plugins_url( 'codemirror/codemirror.min.css', $dir ) );
		wp_enqueue_script( 'codemirror-js', plugins_url( 'codemirror/codemirror.min.js', $dir ), array(), '', true );
		wp_enqueue_script( 'codemirror-css-js', plugins_url( 'codemirror/css.min.js', $dir ), array(), '', true );
}

/**
 * Register settings
 *
 * @todo more options? Like in "which page of your website..."
 */
function hmpl_css_register_settings() {
	register_setting( 'hmpl_css_settings_group', 'hmpl_css_content', 'hmpl_css_validate' );
	register_setting( 'hmpl_css_settings_group', 'hmpl_css_revision', '' );
}

/* 
 * Sanitize :D
 *
 * @todo preprocessor
 * @todo preview
 */
function hmpl_css_validate( $css, $args = array() ) {
	
	$defaults = array(
			'css' => $css, // was: ''
			'is_preview' => false,
			'preprocessor' => '', // Currently not configured
		);
	
	$args = wp_parse_args( $args, $defaults );
	
	$dir = plugin_dir_path( __FILE__ );

	// Remove wp_filter_post_kses, this causes CSS escaping issues
	remove_filter( 'content_save_pre', 'wp_filter_post_kses' );
	remove_filter( 'content_filtered_save_pre', 'wp_filter_post_kses' );
	remove_all_filters( 'content_save_pre' );

	/**
	 * Fires prior to saving custom css values. Necessitated because the
	 * core WordPress save_pre filters were removed:
	 * - content_save_pre
	 * - content_filtered_save_pre
	 *
	 * @since ?
	 * @module Custom_CSS
	 * @see self::save() for proper $args fields
	 * @param array $args See Jetpack_Custom_CSS::save() docblock for more
	 * @todo update variables
	 * @todo add csstidy to this plugin
	 **/
	do_action( 'safecss_save_pre', $args );
	
	/**
	 * @todo figure out where to output errors
	 */
	$warnings = array();

	safecss_class();
	$csstidy = new csstidy();
	$csstidy->optimise = new safecss( $csstidy );

	$csstidy->set_cfg( 'remove_bslash',              false );
	$csstidy->set_cfg( 'compress_colors',            false );
	$csstidy->set_cfg( 'compress_font-weight',       false );
	$csstidy->set_cfg( 'optimise_shorthands',        0 );
	$csstidy->set_cfg( 'remove_last_;',              false );
	$csstidy->set_cfg( 'case_properties',            false );
	$csstidy->set_cfg( 'discard_invalid_properties', true );
	$csstidy->set_cfg( 'css_level',                  'CSS3.0' );
	$csstidy->set_cfg( 'preserve_css',               true );
	$csstidy->set_cfg( 'template',                   $dir . 'csstidy/wordpress-standard.tpl' );
	
	/**
	 * @todo figure out what this is
	 */
	$css = $orig = $args['css'];

	/** 
	 * Corrects invalid color hex.
	 */
	$css = preg_replace( '/\\\\([0-9a-fA-F]{4})/', '\\\\\\\\$1', $prev = $css );

	if ( $css != $prev )
		$warnings[] = 'preg_replace found stuff';

	// Some people put weird stuff in their CSS, KSES tends to be greedy
	$css = str_replace( '<=', '&lt;=', $css );
	// Why KSES instead of strip_tags?  Who knows?
	$css = wp_kses_split( $prev = $css, array(), array() );
	$css = str_replace( '&gt;', '>', $css ); // kses replaces lone '>' with &gt;
	// Why both KSES and strip_tags?  Because we just added some '>'.
	$css = strip_tags( $css );

	if ( $css != $prev )
		$warnings[] = 'kses found stuff';

	// if we're not using a preprocessor
	if ( ! $args['preprocessor'] ) {
		$csstidy->parse( $css );

		$css = $csstidy->print->plain();
	}
	
	/**
	 * Everything is lowercase already at this point of validation, 
	 * use it like this: '@import|@somethingelse|content'
	 *
	 * @todo create array and count
	 * @param $regex_unwanted everything between @param $unwanted and ;
	 */
	$unwanted = '@import';
	
	/** 
	 * It should already be one liners at this point
	 * 
	 * (?:(\/\*)?) 		: catch open comment if exists 	// Group 1
	 * ({$unwanted}.*;) : catch unwanted group 			// Group 2
	 * (?:(\*\/)?) 		: catch close comment if exists // Group 3
	 * 
	 * (?:(\/\*)?)(@import.*;)(?:(\*\/)?)
	 * Testing reference: http://www.regexr.com/
	 */

	$regex_unwanted = "/(?:(\/\*)?)({$unwanted}.*;)(?:(\*\/)?)/";
	
//	$css = preg_replace_callback($regex_unwanted,'hmpl_preg_callback',$css); //php 5.4+ only
	$css = preg_replace($regex_unwanted,"/*$2*/\n\r",$css);
	
	// Save the clean CSS
	return $css;
}

function hmpl_preg_callback($matches) {
	// Preserve line spacing, pushes the second */ 2 lines down though...
	return "/*" . $matches[2] . "*/\n\r";
}

//* calls csstidy
function safecss_class() {
	// Wrapped so we don't need the parent class just to load the plugin
	if ( class_exists('safecss') )
		return;
	
	$dir = plugin_dir_path( __FILE__ );
	
	require_once( $dir . 'csstidy/class.csstidy.php' );

	class safecss extends csstidy_optimise {
		function safecss( &$css ) {
			return $this->csstidy_optimise( $css );
		}

		function postparse() {
			
			/**
			 * Do actions after parsing the css
			 *
			 * @since ?
			 * @module Custom_CSS
			 * @param safecss $obj
			 **/
			do_action( 'csstidy_optimize_postparse', $this );

			return parent::postparse();
		}

		function subvalue() {

			/**
			 * Do action before optimizing the subvalue
			 *
			 * @since ?
			 * @module Custom_CSS
			 * @param safecss $obj
			 **/
			do_action( 'csstidy_optimize_subvalue', $this );

			return parent::subvalue();
		}
	}
}


//* @todo: use this
function hmpl_css_minify( ) {
	
	// Todo: CSS tidy
	
	$element = 'div';
	
	$css = $element . ' {' . $css . '}';
	/** 
	 * Corrects invalid color hex.
	 */
	$css = preg_replace( '/\\\\([0-9a-fA-F]{4})/', '\\\\\\\\$1', $prev = $css );
	
	// Prevent HTML and other unwanted stuff
	$css = wp_kses_split( $css, array(), array() ); // default	

	$css = str_replace( array( "\n","\r","\t" ), '', $css ); // Replace lines, returns and tabs with nothing 
	
	// Space out the data to maintain valid CSS
	preg_match( "/^{$element}\s*{(.*)}\s*$/", $css, $matches );
	
	if ( empty( $matches[1] ) )
			return '';
	
	return $matches[1];
}

/**
 * Render Admin Menu page
 */
function hmpl_css_render_submenu_page() {
	/** 
	 * Cache buster
	 * @int hexadecimal
	 * @param bust adds one to the option
	 * If user tries to change this option illegally, it adds one anyway.
	 */
	$bust = intval( get_option( 'hmpl_css_revision' ), 16 ) + 1;
	$pre_rev = get_option( 'hmpl_css_revision', $bust );
	
	$revision = isset( $pre_rev ) && ! empty( $pre_rev ) ? $pre_rev : '1';
	
	$safe_rev = intval( $_POST['hmpl_css_revision'] );
	if( isset( $_POST['hmpl_css_revision'] ) ) {
		if ( ! $safe_rev  ) {
			$safe_rev = $bust; //add one anyway
		} else if ( $safe_rev > 2147483647 ) {
			$safe_rev = 1; //Start over, clear your CDN cache once every million years :)
		} else {
			$safe_rev = $bust;
		}
		update_option( 'hmpl_css_revision', $safe_rev );
	}
	
	/**
	 * Get CSS
	 */
	$option = get_option('hmpl_css_content');
	$css = isset( $option ) && ! empty( $option ) ? $option : __('/* Enter Your Custom CSS Here */', 'hmpl_css'); // Added text domain
	?>
	
	<div class="wrap">
		<h2 style="margin-bottom: 1em;">
			<span class="dashicons dashicons-desktop" style="vertical-align: top;font-size: 150%;width: auto;"></span> <?php _e( 'Custom CSS', 'hmpl_css' ); ?>
		</h2>
			
		<?php if ( isset( $_GET['settings-updated'] ) ) : ?>
				<div id="message" class="updated"><p><?php _e( 'Custom CSS updated successfully.', 'hmpl_css' ); ?></p></div>
				
			<?php endif; ?>	
		<form name="hmpl_css-form" action="options.php" method="post" enctype="multipart/form-data">		
			<?php settings_fields( 'hmpl_css_settings_group' ); ?>
			<?php // add invisible revision counter ?>
			<input type="hidden" id="hmpl_css_revision" name="hmpl_css_revision" value="<?php echo intval($revision, 10) + 1 ?>" />

			<div id="templateside">
				<p style="margin-top: 0"><?php _e( 'With Custom CSS you can simply include your own style within any theme.', 'hmpl_css' ) ?></p>
				<p><?php _e( 'You can also switch themes and the CSS will be preserved, this makes everything so much easier.', 'hmpl_css' ) ?></p>
				<p><?php _e( 'Be wary when switching themes that not every theme uses the same classes and ID\'s.', 'hmpl_css' ) ?></p>
				<div>
					<?php submit_button( __( 'Publish', 'hmpl_css' ), 'primary', 'submit', true ); ?>
				</div>
				
				<?php // Make these revisions revertable? ?>
				<p>
					<?php _e( 'Revision: ', 'hmpl_css' ) ?>
					<?php echo intval($revision, 10); ?>
				</p>
			</div>
			
			<div id="template">
				<div>
					<textarea cols="70" rows="30" name="hmpl_css_content" id="hmpl_css_content" ><?php echo esc_html( $css ); ?></textarea>
				</div>
				<div>
					<?php submit_button( __( 'Publish', 'hmpl_css' ), 'primary', 'submit', true ); ?>
				</div>
			</div>
			
		</form>
		<script language="javascript">
			jQuery( document ).ready( function() {
				var editor = CodeMirror.fromTextArea( document.getElementById( "hmpl_css_content" ), {lineNumbers: true, lineWrapping: true, mode: 'css' } );
			});
		</script>
				
	</div>
<?php
}