<?php

/**
 * Print the CSS. Either in a link (if more than 2000 chars) or plain text.
 * This helps speeding up the downloading time of your page.
 * This plugin tries to get this file sent to the CDN through W3 Total Cache if set.
 */
function hmpl_css_maybe_print_css() {
	global $blog_id, $current_blog;
	
	if ( ( 1 == $current_blog->spam || 1 == $current_blog->deleted ) )
		return;
	
	$option	= get_option( 'hmpl_css_content' );
	$revision = get_option( 'hmpl_css_revision' );
	
	$css = $option ? $option : '';
	$rev = $revision ? $revision : '';
	
	// Print CSS
	if ( !empty($css) ) {
		// If less than 2000 characters, inline it.
		if ( strlen( $css ) < 2000 ) {
			// Make it readable in source
			$output = '<style type="text/css" id="custom-css" media="screen">' . "\r\n" . $css . "\r\n" . '</style>' . "\r\n";
			echo $output;
		// Else, output it in file
		} else {
			$href = home_url( '/' );
			$href = add_query_arg( 'custom-css', 1, $href ); // set header to be called
			$href = add_query_arg( 'ccblog', $blog_id, $href ); // prevent cross caching, add blog id
			if ( $rev )
				$href = add_query_arg( 'ver', $rev, $href ); // bust cache upon revision change

			$output = sprintf( '<link rel="stylesheet" id="custom-css" type="text/css" href="%s" />' . "\r\n", esc_url_raw( $href ));
			echo $output;
		}
	}
}
add_action( 'wp_head', 'hmpl_css_maybe_print_css' );

//* Needs to be called earlier to prevent header from being output in file
function hmpl_css_maybe_file_css() {
	if ( ( 1 == $current_blog->spam || 1 == $current_blog->deleted ) )
		return;
	
	// at this point the CSS should contain something already
	$css = get_option( 'hmpl_css_content' );	
	
	// Set header for CSS file and echo the css
	if( isset( $_GET['custom-css'] ) && intval( $_GET['custom-css'] ) == 1 ) {
		header( 'Content-Type: text/css', true, 200 );
		header( 'Expires: ' . gmdate( 'D, d M Y H:i:s', time() + 31536000) . ' GMT' ); // 1 year
		echo $css;
		die();
	}
}
add_action( 'plugins_loaded', 'hmpl_css_maybe_file_css' );